<?php

namespace WcFattureGratisInvoice;

if (!defined('ABSPATH')) {
    exit; // Exit if accessed directly.
}

/**
 * Class Autoloader.
 *
 * @see    http://www.php-fig.org/psr/psr-4/
 * @see    https://github.com/php-fig/fig-standards/blob/master/accepted/PSR-4-autoloader-examples.md
 */
final class Autoloader
{
    /**
     * An associative array where the key is a namespace prefix and the value
     * is an array of base directories for classes in that namespace.
     *
     * @var array
     */
    private $prefixes = [];

    /**
     * Load the mapped file for a namespace prefix and relative class.
     *
     * @since  1.0.0
     *
     * @param string $prefix        the namespace prefix
     * @param string $relativeClass the relative class name
     *
     * @return mixed boolean false if no mapped file can be loaded, or the
     *               name of the mapped file that was loaded
     */
    private function loadMappedFile($prefix, $relativeClass)
    {
        // Are there any base directories for this namespace prefix?
        if (false === isset($this->prefixes[$prefix])) {
            return false;
        }

        // Set the base dir path.
        $base_dir = $this->prefixes[$prefix];
        // Remove the prefix from the relative class name.
        $relativeClass = str_replace($prefix, '', $relativeClass);
        // Build the file path.
        $file = $base_dir.str_replace('\\', '/', $relativeClass).'.php';

        // Get the absoluted path of the file class.
        $file = Plugin::getPluginDirPath($file);

        // If the mapped file exists, require it.
        if ($this->requireFile($file)) {
            // Yes, we're done.
            return $file;
        }

        // Never found it.
        return false;
    }

    /**
     * If a file exists, require it from the file system.
     *
     * @since  1.0.0
     *
     * @param string $file the file to require
     *
     * @return bool true if the file exists, false if not
     */
    private function requireFile($file)
    {
        if (file_exists($file)) {
            require $file;

            return true;
        }

        return false;
    }

    /**
     * Register loader with SPL auto-loader stack.
     *
     * @return void
     */
    public function register()
    {
        spl_autoload_register([$this, 'loadClass']);
    }

    /**
     * Add NameSpaces.
     *
     * Add multiple namespaces at once
     *
     * @since  1.0.0
     *
     * @param array $array the list of the namespaces to add
     *
     * @return void
     */
    public function addNamespaces(array $array)
    {
        foreach ($array as $ns => $path) {
            $this->addNamespace($ns, $path);
        }
    }

    /**
     * Adds a base directory for a namespace prefix.
     *
     * @since  1.0.0
     *
     * @param string $prefix  the namespace prefix
     * @param string $baseDir a base directory for class files in the
     *                        namespace
     *
     * @return void
     */
    public function addNamespace($prefix, $baseDir)
    {
        // Normalize namespace prefix.
        $prefix = trim($prefix, '\\').'\\';

        // Normalize the base directory with a trailing separator.
        $baseDir = rtrim($baseDir, DIRECTORY_SEPARATOR).'/';

        // Initialize the namespace prefix array.
        if (false === isset($this->prefixes[$prefix])) {
            $this->prefixes[$prefix] = [];
        }

        $this->prefixes[$prefix] = $baseDir;
    }

    /**
     * Loads the class file for a given class name.
     *
     * @since  1.0.0
     *
     * @param string $class the fully-qualified class name
     *
     * @return mixed the mapped file name on success, or boolean false on
     *               failure
     */
    public function loadClass($class)
    {
        $mapped = false;

        foreach ($this->prefixes as $prefix => $path) {
            if (false !== strpos($class, $prefix)) {
                $mapped = $this->loadMappedFile($prefix, $class);
                break;
            }
        }

        return $mapped;
    }
}
