<?php

namespace WcFattureGratisInvoice\EndPoint;

if (!defined('ABSPATH')) {
    exit; // Exit if accessed directly.
}

use WcFattureGratisInvoice\Functions as F;

class Endpoints implements EndpointsInterface
{
    /**
     * Api endpoint.
     *
     * @var string
     */
    const ENDPOINT = 'wc-inv';

    /**
     * Query var.
     *
     * @since 1.0.0
     *
     * @var string
     */
    public $var;

    /**
     * Post Type.
     *
     * @since 1.0.0
     *
     * @var string The post type for this json
     */
    public $postType = ['shop_order'];

    private const CONTENT_TYPE = [
        'pdf' => 'application/pdf',
        'xml' => 'application/xml',
        'info' => 'text/plain',
    ];

    /**
     * JsonEndpoint constructor.
     */
    public function __construct()
    {
        $this->var = 'wc-fatture-gratis-inv';
    }

    /**
     * Add EndPoint.
     *
     * @return null
     */
    public function addEndPoint()
    {
        // Esc if empty post type & endpoint
        if (null === $this->postType && null === self::ENDPOINT) {
            return null;
        }

        if (is_array($this->postType) && sizeof($this->postType) >= 0) {
            // Multi post type
            foreach ($this->postType as $type) {
                $this->addRewriteRule(self::ENDPOINT, (string) $type);
            }
        } else {
            $this->addRewriteRule(self::ENDPOINT, (string) $this->postType);
        }

        flush_rewrite_rules();
    }

    /**
     * Add Query Vars.
     *
     * @since 1.0.0
     *
     * @param $vars
     *
     * @return array
     */
    public function addQueryVars($vars)
    {
        $vars[] = $this->var;

        return $vars;
    }

    /**
     * Add Rewrite Rule.
     *
     * @since 1.0.0
     *
     * @param $endpoint
     * @param $postType
     */
    public function addRewriteRule($endpoint, $postType)
    {
        // Add new rewrite tags to WP for our endpoint's post_type
        add_rewrite_tag('%'.$postType.'%', '([^&]+)');

        // and post_id arguments
        add_rewrite_tag('%'.$postType.'_id%', '([0-9]+)');

        // general rule finds "all" (post_per_page) of a given post_type
        add_rewrite_rule(
            $endpoint.'/([^&]+)/?$',
            'index.php?'.$postType.'=$matches[1]&'.$this->var.'=yes',
            'top'
        );

        // specific rule finds a single post
        add_rewrite_rule(
            $endpoint.'/([^&]+)/([0-9]+)/?$',
            'index.php?'.$postType.'=$matches[1]&'.$postType.'_id=$matches[2]&'.$this->var.'=yes',
            'top'
        );
    }

    /**
     * Add Rewrite Endpoint.
     *
     * @since 1.0.0
     */
    public function addRewriteEndpoint()
    {
        add_rewrite_endpoint(self::ENDPOINT, EP_PAGES);
    }

    /**
     * Set Query Args.
     *
     * @since 1.0.0
     *
     * @return null
     */
    public function setQueryArgs()
    {
        $query = F\getWpQuery();

        $format = F\filterInput($_GET, 'format', FILTER_SANITIZE_STRING);

        if (\in_array($format, ['pdf', 'xml', 'info'])) {
            $uuid = F\filterInput($_GET, 'uuid', FILTER_SANITIZE_STRING);
            if ($uuid) {
                $this->fileDownload($format, $uuid);
            }
        }

        if (!$query) {
            return null;
        }

        // Set format
        if ($format) {
            $query->set('format', $format);
        }
    }

    private function fileDownload(string $format, string $uuid): void
    {
        $doc_path = __DIR__."/../../files/$format/".$uuid.".$format";
        header('Content-Type: '.self::CONTENT_TYPE[$format]);
        header(sprintf('Content-Disposition: attachment; filename="%s.%s"', $uuid, $format));
        header('Content-Transfer-Encoding: binary');
        header('Content-Length: '.filesize($doc_path));
        header('Accept-Ranges: bytes');
        readfile($doc_path) or exit('Unable to open file!');
        exit();
    }
}
