<?php

namespace WcFattureGratisInvoice\Functions;

if (!defined('ABSPATH')) {
    exit; // Exit if accessed directly.
}

use WcFattureGratisInvoice\Plugin;
use WcFattureGratisInvoice\Sanitize\Arrays;
use WcFattureGratisInvoice\Sanitize\Text;

/**
 * Filter Input.
 *
 * @since 1.0.0
 *
 * @uses  filter_var() To filter the value.
 *
 * @param array  $data    the haystack of the elements
 * @param string $key     the key of the element within the haystack to filter
 * @param int    $filter  the filter to use
 * @param array  $options the option for the filter var
 *
 * @return bool|mixed the value filtered on success false if filter fails or key doesn't exists
 */
function filterInput($data, $key, $filter = FILTER_DEFAULT, $options = [])
{
    return isset($data[$key]) ? filter_var($data[$key], $filter, $options) : false;
}

/**
 * Strip Content.
 *
 * @since 1.0.0
 *
 * @param      $string
 * @param bool $remove_breaks
 *
 * @return string|string[]|null
 */
function stripTags($string, $remove_breaks = false)
{
    // Strip tags
    $string = strip_tags($string);
    // Clean up things like &amp;
    $string = html_entity_decode($string, ENT_COMPAT, 'UTF-8');
    // Replace Multiple spaces with single space
    $string = preg_replace('/ +/', ' ', $string);
    // Strip shortcode
    $string = rtrim(strip_shortcodes($string), "\n\t\r");
    // Strip images.
    $string = preg_replace('/<img[^>]+\>/i', '', $string);
    // Strip div.
    $string = preg_replace("/<div>(.*?)<\/div>/", '$1', $string);
    // Strip scripts.
    $string = preg_replace('@<(script|style)[^>]*?>.*?</\\1>@si', '', $string);
    // Convert € symbol
    $string = str_replace('€', '[EURO]', $string);
    // Convert & symbol
    $string = str_replace('&', 'E', $string);
    // Remove illegal charset
    $string = str_replace(['<', '>', '"', "'"], '', $string);
    // Convert dash
    $string = str_replace(['-', '_'], ' ', $string);
    // Convert per cent
    $string = str_replace('%', '[PERCENT]', $string);
    // Convert accents
    $string = str_replace(['à', 'á'], 'a', $string);
    $string = str_replace(['é', 'è'], 'e', $string);
    $string = str_replace(['ì', 'í'], 'i', $string);
    $string = str_replace(['ò', 'ó'], 'o', $string);
    $string = str_replace(['ù', 'ú'], 'u', $string);

    if ($remove_breaks) {
        $string = preg_replace('/\s+/', ' ', $string);
        $string = preg_replace('/[\r\n\t ]+/', ' ', $string);
    }

    return trim($string);
}

/**
 * Sanitize function.
 *
 * @since 1.0.0
 *
 * @param $input string|array The string or array to sanitize
 *
 * @return array|bool|string Sanitized value text or array, otherwise false
 */
function sanitize($input)
{
    if (is_string($input)) {
        try {
            return Text::sanitize($input);
        } catch (\Exception $e) {
            echo 'Input is not a string: ', $e->getMessage(), "\n";
        }
    } elseif (is_int($input)) {
        try {
            return $input;
        } catch (\Exception $e) {
            echo 'Input is not a int value: ', $e->getMessage(), "\n";
        }
    } elseif (is_array($input)) {
        try {
            return Arrays::sanitize($input);
        } catch (\Exception $e) {
            echo 'Input is not a array: ', $e->getMessage(), "\n";
        }
    } else {
        return false;
    }
}

/**
 * WpMl Switch language.
 *
 * @since 1.0.0
 */
function switchLang()
{
    if (isWpmlActive() && defined('ICL_LANGUAGE_CODE')) {
        global $sitepress;
        $sitepress->switch_lang(ICL_LANGUAGE_CODE);
    }
}

/**
 * Get current lang.
 *
 * @since 1.0.0
 *
 * @return string
 */
function getCurrentLanguage()
{
    $lang = '';
    $locale = substr(get_locale(), 0, -3);

    if ($locale) {
        $lang = $locale;
    }

    if (isWpmlActive()) {
        $lang = defined('ICL_LANGUAGE_CODE') ? ICL_LANGUAGE_CODE : '';
    }

    return $lang;
}

/**
 * Send a JSON response back to an Ajax request, indicating failure.
 *
 * If the `$data` parameter is a WP_Error object, the errors
 * within the object are processed and output as an array of error
 * codes and corresponding messages. All other types are output
 * without further processing.
 *
 * @since 1.0.0
 *
 * @param mixed $data        data to encode as JSON, then print and die
 * @param int   $status_code the HTTP status code to output
 */
function sendJsonError($data = null, $status_code = null)
{
    $response = ['success' => false];

    if (isset($data)) {
        if (is_wp_error($data)) {
            $result = [];
            foreach ($data->errors as $code => $messages) {
                foreach ($messages as $message) {
                    $result[] = ['code' => $code, 'message' => $message];
                }
            }

            $response['data'] = $result;
        } else {
            $response['data'] = $data;
        }
    }

    sendJson($response, $status_code);
}

/**
 * Get Customers List.
 *
 * @since 1.0.0
 *
 * @return array The customer list
 */
function getCustomersList()
{
    $users = get_users();
    $userList = [];

    if (!empty($users) && !is_wp_error($users)) {
        foreach ($users as $user) {
            $role = !empty($user->roles) ? ucfirst(" - {$user->roles[0]}") : '';
            $userList[$user->data->ID] = "{$user->data->display_name}{$role}";
        }
    }

    return $userList;
}

/**≤
 * Send a JSON response back to an Ajax request.
 *
 * @since 1.0.0
 *
 * @param mixed $response    Variable (usually an array or object) to encode as JSON,
 *                           then print and die.
 * @param int   $status_code The HTTP status code to output.
 */
function sendJson($response, $status_code = null)
{
    @header('Content-Type: application/json; charset='.get_option('blog_charset'));

    if (null !== $status_code) {
        status_header($status_code);
    }

    echo wp_json_encode($response);

    if (wp_doing_ajax()) {
        wp_die('', '', [
            'response' => null,
        ]);
    } else {
        exit;
    }
}

/**
 * Get Post Meta.
 *
 * @param      $key
 * @param null $default
 * @param int  $post
 * @param bool $single
 *
 * @return mixed|null
 */
function getPostMeta($key, $default = null, $post = 0, $single = true)
{
    // Get the post.
    $post = get_post($post);

    if (!$post) {
        return $default;
    }

    // Return the default value if meta data doesn't exists.
    if (!metadata_exists('post', $post->ID, $key) && null !== $default) {
        return $default;
    }

    // Retrieve the post meta.
    return get_post_meta($post->ID, $key, $single);
}

/**
 * Disable Plugin.
 *
 * This function disable the plugin because of his dependency.
 *
 * @since 1.0.0
 *
 * @return void
 */
function disablePlugin()
{
    if (!function_exists('deactivate_plugins')) {
        require_once untrailingslashit(ABSPATH).'/wp-admin/includes/plugin.php';
    }

    if (!isWooCommerceActive()) {
        add_action('admin_notices', function () {
            ?>
            <div class="notice notice-error">
                <p><span class="dashicons dashicons-no"></span>
                    <?php esc_html_e(
                'FattureGratis -> (Invoice) has been deactivated. The plugin require: WooCommerce',
                WC_FATTUREGRATIS_INV_TEXTDOMAIN
            ); ?>
                </p>
            </div>
            <?php

            // Don't show the activated notice.
            if (isset($_GET['activate'])) {
                unset($_GET['activate']);
            }
        });

        // Deactivate the plugin.
        deactivate_plugins(WC_FATTUREGRATIS_INV_PLUGIN_DIR.'/index.php');
    }
}
